==> bgpy_pkg-4.0.2-py3-none-any.whl: bgpy/tests/engine_tests/engine_test_configs/internals/internal_config_000.py <==
from frozendict import frozendict
from copy import deepcopy

from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.as_graphs import PeerLink, CustomerProviderLink as CPLink
from bgpy.as_graphs import ASGraphInfo

from bgpy.enums import SpecialPercentAdoptions
from bgpy.simulation_engine import BGPPolicy, BaseSimulationEngine
from bgpy.simulation_framework import (
    ValidPrefix,
    ScenarioConfig,
)
from bgpy.enums import Prefixes


as_graph_info = ASGraphInfo(
    peer_links=frozenset([PeerLink(1, 3), PeerLink(1, 5)]),
    customer_provider_links=frozenset(
        [
            CPLink(provider_asn=2, customer_asn=1),
            CPLink(provider_asn=1, customer_asn=4),
        ]
    ),
)


class Custom00ValidPrefix(ValidPrefix):
    """Add a better announcement in round 2 to cause withdrawal"""

    def post_propagation_hook(
        self,
        engine: "BaseSimulationEngine",
        percent_adopt: float | SpecialPercentAdoptions,
        trial: int,
        propagation_round: int,
    ) -> None:
        if propagation_round == 1:  # second round
            ann = deepcopy(
                engine.as_graph.as_dict[2].policy._local_rib.get(Prefixes.PREFIX.value)
            )
            # Add a new announcement at AS 3, which will be better than the one
            # from 2 and cause a withdrawn route by 1 to 4
            # ann.seed_asn = 3
            # ann.as_path = (3,)
            object.__setattr__(ann, "seed_asn", 3)
            object.__setattr__(
                ann,
                "as_path",
                (3,),
            )
            engine.as_graph.as_dict[3].policy._local_rib.add_ann(ann)
            Custom00ValidPrefix.victim_asns = frozenset({2, 3})
            self.victim_asns: frozenset[int] = frozenset({2, 3})


internal_config_000 = EngineTestConfig(
    name="internal_000",
    desc="Test withdrawal mechanism caused by better announcement",
    scenario_config=ScenarioConfig(
        ScenarioCls=Custom00ValidPrefix,
        BasePolicyCls=BGPPolicy,
        override_victim_asns=frozenset({2}),
        override_non_default_asn_cls_dict=frozendict(),
    ),
    as_graph_info=as_graph_info,
    propagation_rounds=3,
)

==> bgpy_pkg-7.0.3-py3-none-any.whl: bgpy/tests/engine_tests/engine_test_configs/internals/internal_config_000.py <==
from frozendict import frozendict
from copy import deepcopy

from bgpy.tests.engine_tests.utils import EngineTestConfig

from bgpy.as_graphs import PeerLink, CustomerProviderLink as CPLink
from bgpy.as_graphs import ASGraphInfo

from bgpy.enums import SpecialPercentAdoptions
from bgpy.simulation_engine import BGPFull, BaseSimulationEngine
from bgpy.simulation_framework import (
    ValidPrefix,
    ScenarioConfig,
)
from bgpy.enums import Prefixes


as_graph_info = ASGraphInfo(
    peer_links=frozenset([PeerLink(1, 3), PeerLink(1, 5)]),
    customer_provider_links=frozenset(
        [
            CPLink(provider_asn=2, customer_asn=1),
            CPLink(provider_asn=1, customer_asn=4),
        ]
    ),
)


class Custom00ValidPrefix(ValidPrefix):
    """Add a better announcement in round 2 to cause withdrawal"""

    min_propagation_rounds: int = 3

    def post_propagation_hook(
        self,
        engine: "BaseSimulationEngine",
        percent_adopt: float | SpecialPercentAdoptions,
        trial: int,
        propagation_round: int,
    ) -> None:
        if propagation_round == 1:  # second round
            ann = deepcopy(
                engine.as_graph.as_dict[2].policy._local_rib.get(Prefixes.PREFIX.value)
            )
            # Add a new announcement at AS 3, which will be better than the one
            # from 2 and cause a withdrawn route by 1 to 4
            # ann.seed_asn = 3
            # ann.as_path = (3,)
            object.__setattr__(ann, "seed_asn", 3)
            object.__setattr__(
                ann,
                "as_path",
                (3,),
            )
            engine.as_graph.as_dict[3].policy._local_rib.add_ann(ann)
            Custom00ValidPrefix.victim_asns = frozenset({2, 3})
            self.victim_asns: frozenset[int] = frozenset({2, 3})


internal_config_000 = EngineTestConfig(
    name="internal_000",
    desc="Test withdrawal mechanism caused by better announcement",
    scenario_config=ScenarioConfig(
        ScenarioCls=Custom00ValidPrefix,
        BasePolicyCls=BGPFull,
        override_victim_asns=frozenset({2}),
        override_non_default_asn_cls_dict=frozendict(),
    ),
    as_graph_info=as_graph_info,
)
